@@ -224,12 +224,19 @@ class ProxyManager:
                 future.cancel()
 
         # wait for cancelled tasks and handle exceptions
-        for future in asyncio.as_completed(self._running_futures):
+        for future in self._running_futures:
+            if not future.done():
+                future.cancel()
+
             try:
                 await future
+            except asyncio.CancelledError:
+                # ignore cancelled error
+                continue
             except Exception as exc:
                 on_exc(None, exc)
                 continue
+
         self._running_futures = set()
 
         # return the result of the successful task
